<?php

/**
 * @file
 * Avalara Sales Order Processing - sample code.
 */

/**
 * Posts and commits orders in the AvaTax system.
 *
 * @param object $order
 *   The current order object.
 *
 * @return bool
 *   FALSE if the status could not be changed or AvaTax updated
 */

/**
 * Implements hook_order().
 */
function uc_tax_avatax_order($op, &$arg1, $arg2) {

  // Required for Avalara Scripts.
  require_once 'ava_tax.php';
  require_once 'credentials.php';

  // Create the Avatax Company key.
  $company_code = (variable_get('uc_tax_avatax_company_code', ''));
  $use_mode = (variable_get('uc_tax_avatax_use_mode', ''));

  switch ($op) {
    case 'can_update':
      if (variable_get('uc_tax_avatax_billing_address', '')) {
        $zone = $arg1->billing_zone;
      }
      else {
        $zone = $arg1->delivery_zone;
      }

      $state = uc_get_zone_code($zone);

      $avatax_states = array();
      $avatax_states_str = variable_get('uc_tax_avatax_select_states', '');
      if ($avatax_states_str) {
        $avatax_states = explode(", ", $avatax_states_str);
        if (!in_array($state, $avatax_states)) {
          break;
        }
      }

      $new_state = uc_order_status_data($arg2, 'state');
      $client = new TaxServiceSoap($use_mode);
      $request = new GetTaxHistoryRequest();
      $request->setDocCode('uc-' . $arg1->order_id . '');
      $request->setCompanyCode($company_code);
      $request->setDocType(DocumentType::$SalesInvoice);
      $request->setDetailLevel(DetailLevel::$Document);

      // Get the AvatAx status and amount information.
      try {
        $result = $client->getTaxHistory($request);
        if ($result->getResultCode() == SeverityLevel::$Success) {
          $ava_doc_status = $result->getGetTaxResult()->getDocStatus();
          $ava_total_amount = $result->getGetTaxResult()->getTotalAmount();
          $ava_total_tax = $result->getGetTaxResult()->getTotalTax();
        }
        elseif ($result->getResultCode() != SeverityLevel::$Success) {
          foreach ($result->getMessages() as $msg) {
            drupal_set_message(filter_xss('AvaTax error, message is: ' . $msg->getName() . ': ' . $msg->getSummary() . ''), 'error');
          }
          drupal_set_message(filter_xss('Avatax system error, status not updated, refer to administrator.'), 'error');
          return FALSE;
          break;
        }
      }
      catch (SoapFault $exception) {
        $msg = 'SOAP Exception: ';
        if ($exception) {
          $msg .= $exception->faultstring;
        }
        drupal_set_message(filter_xss('AvaTax message is: ' . $msg . '.'), 'error');
        drupal_set_message(filter_xss('AvaTax last request is: ' . $client->__getLastRequest() . ''), 'error');
        drupal_set_message(filter_xss('AvaTax last response is: ' . $client->__getLastResponse() . ''), 'error');
        drupal_set_message(filter_xss('Avatax system error, status not updated, refer to administrator.'), 'error');
        return FALSE;
        break;
      }

      // Block posted or committed or voided orders new state to in_checkout.
      if ($new_state == 'in_checkout') {
        switch ($ava_doc_status) {
          case 'Posted':
            drupal_set_message(filter_xss('AvaTax order state is: ' . $ava_doc_status . '.'), 'error');
            drupal_set_message(filter_xss('Error, Order Posted, status not updated, refer to administrator.'), 'error');
            return FALSE;

          break;

          case 'Committed':
            drupal_set_message(filter_xss('AvaTax order state is: ' . $ava_doc_status . ''), 'error');
            drupal_set_message(filter_xss('Error, Order Committed for Tax submission, status not updated, refer to administrator.'), 'error');
            return FALSE;

          break;

          case 'Cancelled':
            drupal_set_message(filter_xss('AvaTax completed order state is: ' . $ava_doc_status . ''), 'error');
            drupal_set_message(filter_xss('Error, Order Voided, refer to administrator.'), 'error');
            return FALSE;

          break;
        }
        break;
      }

      // Block committed or voided orders new state to post_checkout.
      if ($new_state == 'post_checkout') {
        switch ($ava_doc_status) {
          case 'Committed':
            drupal_set_message(filter_xss('AvaTax order state is: ' . $ava_doc_status . ''), 'error');
            drupal_set_message(filter_xss('Error, Order Committed for Tax submission, status not updated, refer to administrator.'), 'error');
            return FALSE;

          break;

          case 'Cancelled':
            drupal_set_message(filter_xss('AvaTax completed order state is: ' . $ava_doc_status . ''), 'error');
            drupal_set_message(filter_xss('Error, Order Voided, refer to administrator.'), 'error');
            return FALSE;

          break;
        }
        break;
      }

      // Post AvaTax - unless already posted or committed or voided.
      if ($new_state == 'payment_received') {
        switch ($ava_doc_status) {
          case 'Posted':
            drupal_set_message(filter_xss('AvaTax order state is: ' . $ava_doc_status . ''), 'error');
            drupal_set_message(filter_xss('Error, Order Posted, status not updated, refer to administrator.'), 'error');
            return FALSE;

          break;

          case 'Committed':
            drupal_set_message(filter_xss('AvaTax order state is: ' . $ava_doc_status . ''), 'error');
            drupal_set_message(filter_xss('Error, Order Committed for Tax submission, status not updated, refer to administrator.'), 'error');
            return FALSE;

          break;

          case 'Cancelled':
            drupal_set_message(filter_xss('AvaTax completed order state is: ' . $ava_doc_status . ''), 'error');
            drupal_set_message(filter_xss('Error, Order Voided, refer to administrator.'), 'error');
            return FALSE;

          break;

          case 'Saved':
            if (round((uc_payment_balance($arg1)), 2) == 0 && round(($ava_total_amount + $ava_total_tax - $arg1->order_total), 2) == 0) {
              $client = new TaxServiceSoap($use_mode);
              $request = new PostTaxRequest();

              // Locate Document by Invoice Number.
              $request->setDocCode('uc-' . $arg1->order_id . '');
              $date_time = new DateTime();
              $request->setDocDate(date_format($date_time, "Y-m-d"));
              $request->setDocType('SalesInvoice');
              $request->setCompanyCode($company_code);
              $request->setTotalAmount($ava_total_amount);
              $request->setTotalTax($ava_total_tax);

              try {
                $result = $client->postTax($request);
                if ($result->getResultCode() == SeverityLevel::$Success) {
                  break;
                }
                elseif ($result->getResultCode() != SeverityLevel::$Success) {
                  foreach ($result->getMessages() as $msg) {
                    drupal_set_message(filter_xss('AvaTax error, message is: ' . $msg->getName() . ': ' . $msg->getSummary() . ''), 'error');
                  }
                  drupal_set_message(filter_xss('Avatax system error, status not updated, refer to administrator.'), 'error');
                  return FALSE;
                  break;
                }
              }
              catch (SoapFault $exception) {
                $msg = 'SOAP Exception: ';
                if ($exception) {
                  $msg .= $exception->faultstring;
                }
                drupal_set_message(filter_xss('AvaTax message is: ' . $msg . '.'), 'error');
                drupal_set_message(filter_xss('AvaTax last request is: ' . $client->__getLastRequest() . ''), 'error');
                drupal_set_message(filter_xss('AvaTax last response is: ' . $client->__getLastResponse() . ''), 'error');
                drupal_set_message(filter_xss('Avatax system error, status not updated, refer to administrator.'), 'error');
                return FALSE;
                break;
              }
            }
            elseif (round((uc_payment_balance($arg1)), 2) != 0) {
              drupal_set_message(filter_xss('Ubercart order has a balance of: ' . uc_payment_balance($arg1) . ''), 'error');
              drupal_set_message(filter_xss('Payment balance is not zero, status not updated, refer to administrator.'), 'error');
              return FALSE;
              break;
            }
            else {
              drupal_set_message(filter_xss('AvaTax order value is: ' . ($ava_total_amount + $ava_total_tax) . ''), 'error');
              drupal_set_message(filter_xss('Ubercart order value is: ' . $arg1->order_total . '.'), 'error');
              drupal_set_message(filter_xss('Error, Ubercart and AvaTax out of balance, status not updated, refer to administrator.'), 'error');
              return FALSE;
              break;
            }
        }
        break;
      }

      // Commit AvaTax - unless saved or committed or voided.
      if ($new_state == 'completed') {
        switch ($ava_doc_status) {
          case 'Saved':
            drupal_set_message(filter_xss('AvaTax message is: ' . $ava_doc_status . ''), 'error');
            drupal_set_message(filter_xss('Error, Order must first be posted, status not updated, refer to administrator.'), 'error');
            return FALSE;

          break;

          case 'Posted':
            if (round((uc_payment_balance($arg1)), 2) == 0 && round(($ava_total_amount + $ava_total_tax - $arg1->order_total), 2) == 0) {
              $client = new TaxServiceSoap($use_mode);
              $request = new CommitTaxRequest();

              // Locate Document by Invoice Number.
              $request->setDocCode('uc-' . $arg1->order_id . '');
              $request->setDocType('SalesInvoice');
              $request->setCompanyCode($company_code);

              try {
                $result = $client->commitTax($request);
                if ($result->getResultCode() == SeverityLevel::$Success) {
                  break;
                }
                elseif ($result->getResultCode() != SeverityLevel::$Success) {
                  foreach ($result->getMessages() as $msg) {
                    drupal_set_message(filter_xss('AvaTax error, message is: ' . $msg->getName() . ': ' . $msg->getSummary() . ''), 'error');
                  }
                  drupal_set_message(filter_xss('Avatax system error, status not updated, refer to administrator.'), 'error');
                  return FALSE;
                  break;
                }
              }
              catch (SoapFault $exception) {
                $msg = 'SOAP Exception: ';
                if ($exception) {
                  $msg .= $exception->faultstring;
                }
                drupal_set_message(filter_xss('AvaTax message is: ' . $msg . '.'), 'error');
                drupal_set_message(filter_xss('AvaTax last request is: ' . $client->__getLastRequest() . ''), 'error');
                drupal_set_message(filter_xss('AvaTax last response is: ' . $client->__getLastResponse() . ''), 'error');
                drupal_set_message(filter_xss('Avatax system error, status not updated, refer to administrator.'), 'error');
                return FALSE;
                break;
              }
            }
            elseif (round((uc_payment_balance($arg1)), 2) != 0) {
              drupal_set_message(filter_xss('Ubercart order has a balance of: ' . uc_payment_balance($arg1) . ''), 'error');
              drupal_set_message(filter_xss('Payment balance is not zero, status not updated, refer to administrator.'), 'error');
              return FALSE;
              break;
            }
            else {
              drupal_set_message(filter_xss('AvaTax order value is: ' . ($ava_total_amount + $ava_total_tax) . ''), 'error');
              drupal_set_message(filter_xss('Ubercart order value is: ' . $arg1->order_total . '.'), 'error');
              drupal_set_message(filter_xss('Error, Ubercart and AvaTax out of balance, status not updated, refer to administrator.'), 'error');
              return FALSE;
              break;
            }

          case 'Committed':
            drupal_set_message(filter_xss('AvaTax order state is: ' . $ava_doc_status . ''), 'error');
            drupal_set_message(filter_xss('Error, Order Committed for Tax submission, status not updated, refer to administrator.'), 'error');
            return FALSE;

          break;

          case 'Cancelled':
            drupal_set_message(filter_xss('AvaTax completed order state is: ' . $ava_doc_status . ''), 'error');
            drupal_set_message(filter_xss('Error, Order Voided, refer to administrator.'), 'error');
            return FALSE;

          break;
        }
        break;
      }

      // Cancel AvaTax - unless Posted or Committed or Canceled.
      if ($new_state == 'canceled') {
        switch ($ava_doc_status) {
          case 'Saved':
            $client = new TaxServiceSoap($use_mode);
            $request = new CancelTaxRequest();

            // Locate Document by Invoice Number.
            $request->setDocCode('uc-' . $arg1->order_id . '');
            $request->setDocType('SalesInvoice');
            $request->setCompanyCode($company_code);

            // Set type of cancel to delete.
            $code = CancelCode::$DocVoided;
            $request->setCancelCode($code);

            try {
              $result = $client->cancelTax($request);
              if ($result->getResultCode() == 'Success') {
                break;
              }
              elseif ($result->getResultCode() != SeverityLevel::$Success) {
                foreach ($result->getMessages() as $msg) {
                  drupal_set_message(filter_xss('AvaTax error, message is: ' . $msg->getName() . ': ' . $msg->getSummary() . '.'), 'error');
                }
                drupal_set_message(filter_xss('Avatax system error, status not updated, refer to administrator.'), 'error');
                return FALSE;
                break;
              }
            }
            catch (SoapFault $exception) {
              $msg = 'SOAP Exception: ';
              if ($exception) {
                $msg .= $exception->faultstring;
              }
              drupal_set_message(filter_xss('AvaTax message is: ' . $msg . '.'), 'error');
              drupal_set_message(filter_xss('AvaTax last request is: ' . $client->__getLastRequest() . ''), 'error');
              drupal_set_message(filter_xss('AvaTax last response is: ' . $client->__getLastResponse() . ''), 'error');
              drupal_set_message(filter_xss('Avatax system error, status not updated, refer to administrator.'), 'error');
              return FALSE;
              break;
            }

          case 'Posted':
            drupal_set_message(filter_xss('AvaTax order state is: ' . $ava_doc_status . '.'), 'error');
            drupal_set_message(filter_xss('Error, Order Posted for Tax submission, status not updated, refer to administrator.'), 'error');
            return FALSE;

          break;

          case 'Committed':
            drupal_set_message(filter_xss('AvaTax order state is: ' . $ava_doc_status . '.'), 'error');
            drupal_set_message(filter_xss('Error, Order Committed for Tax submission, status not updated, refer to administrator.'), 'error');
            return FALSE;

          break;

          case 'Cancelled':
            drupal_set_message(filter_xss('AvaTax completed order state is: ' . $ava_doc_status . '.'), 'error');
            drupal_set_message(filter_xss('Error, Order Voided, refer to administrator.'), 'error');
            return FALSE;

          break;
        }
        break;
      }

    case 'delete':
      if (variable_get('uc_tax_avatax_billing_address', '')) {
        $zone = $arg1->billing_zone;
      }
      else {
        $zone = $arg1->delivery_zone;
      }

      $state = uc_get_zone_code($zone);

      $avatax_states = array();
      $avatax_states_str = variable_get('uc_tax_avatax_select_states', '');
      if ($avatax_states_str) {
        $avatax_states = explode(", ", $avatax_states_str);
        if (!in_array($state, $avatax_states)) {
          break;
        }
      }

      $client = new TaxServiceSoap($use_mode);
      $request = new CancelTaxRequest();

      // Locate Document by Invoice Number.
      $request->setDocCode('uc-' . $arg1->order_id . '');
      $request->setDocType('SalesInvoice');
      $request->setCompanyCode($company_code);

      // Set type of cancel to delete.
      $code = CancelCode::$DocVoided;
      $request->setCancelCode($code);

      try {
        $result = $client->cancelTax($request);
        if ($result->getResultCode() == 'Success') {
          break;
        }
        elseif ($result->getResultCode() != 'Success') {
          drupal_set_message(filter_xss('AvaTax does not have a record of this order, please contact your administrator.'), 'warning');
          return FALSE;
          break;
        }
      }
      catch (SoapFault $exception) {
        $msg = 'SOAP Exception: ';
        if ($exception) {
          $msg .= $exception->faultstring;
        }
        drupal_set_message(filter_xss('AvaTax message is: ' . $msg . '.'), 'error');
        drupal_set_message(filter_xss('AvaTax last request is: ' . $client->__getLastRequest() . ''), 'error');
        drupal_set_message(filter_xss('AvaTax last response is: ' . $client->__getLastResponse() . ''), 'error');
        drupal_set_message(filter_xss('Avatax system error, status not updated, refer to administrator.'), 'error');
      }

  }
}
